summaryrefslogtreecommitdiff
path: root/db2/db
diff options
context:
space:
mode:
authorUlrich Drepper <drepper@redhat.com>1997-11-12 00:06:02 +0000
committerUlrich Drepper <drepper@redhat.com>1997-11-12 00:06:02 +0000
commitcc3fa755128c70f8afc2d604936f33e4d7d652d8 (patch)
tree0c68c1298140e1524078e8bfdf5154fb6129b680 /db2/db
parent86187531d302de284b67ac162cf76c60c86bd7da (diff)
1997-11-11 21:30 Ulrich Drepper <drepper@cygnus.com> * include/sys/stat.h: Define stat, fstat, lstat and *64 variants as macros so the the library compiles correctly even without optimization. * io/fstat.c: Undef fstat. * io/fstat64.c: Undef fstat64 * io/lstat.c: Undef lstat. * io/lstat64.c: Undef lstat64 * io/stat.c: Undef stat. * io/stat64.c: Undef stat64 * io/fts.c: Include <include/sys/stat.h> to get macro definitions. * io/ftw.c: Likewise. * io/getdirname.c: Likewise. * Makefile (install): Run test-installation.pl if possible. * db2/Makefile: Update from db-2.3.12. * db2/db.h: Likewise. * db2/db_int.h: Likewise. * db2/btree/bt_cursor.c: Likewise. * db2/btree/bt_delete.c: Likewise. * db2/btree/bt_open.c: Likewise. * db2/btree/bt_put.c: Likewise. * db2/btree/bt_rec.c: Likewise. * db2/btree/bt_recno.c: Likewise. * db2/btree/bt_search.c: Likewise. * db2/btree/bt_split.c: Likewise. * db2/btree/bt_stat.c: Likewise. * db2/btree/btree.src: Likewise. * db2/btree/btree_auto.c: Likewise. * db2/btree/bt_cursor.c: Likewise. * db2/btree/bt_delete.c: Likewise. * db2/btree/bt_open.c: Likewise. * db2/btree/bt_put.c: Likewise. * db2/btree/bt_rec.c: Likewise. * db2/btree/bt_recno.c: Likewise. * db2/btree/bt_search.c: Likewise. * db2/btree/bt_split.c: Likewise. * db2/btree/bt_stat.c: Likewise. * db2/btree/btree.src: Likewise. * db2/btree/btree_auto.c: Likewise. * db2/common/db_appinit.c: Likewise. * db2/common/db_apprec.c: Likewise. * db2/common/db_byteorder.c: Likewise. * db2/common/db_region.c: Likewise. * db2/db/db.c: Likewise * db2/db/db.src: Likewise * db2/db/db_auto.c: Likewise * db2/db/db_dispatch.c: Likewise * db2/db/db_dup.c: Likewise * db2/db/db_overflow.c: Likewise * db2/db/db_pr.c: Likewise * db2/db/db_rec.c: Likewise * db2/db/db_ret.c: Likewise * db2/db/db_thread.c: Likewise * db2/db185/db185.c: Likewise. * db2/hash/hash.c: Likewise. * db2/hash/hash.src: Likewise. * db2/hash/hash_auto.c: Likewise. * db2/hash/hash_dup.c: Likewise. * db2/hash/hash_page.c: Likewise. * db2/hash/hash_rec.c: Likewise. * db2/include/btree_auto.h: Likewise. * db2/include/btree_ext.h: Likewise. * db2/include/clib_ext.h: Likewise. * db2/include/common_ext.h: Likewise. * db2/include/db.h.src: Likewise. * db2/include/db_am.h: Likewise. * db2/include/db_auto.h: Likewise. * db2/include/db_cxx.h: Likewise. * db2/include/db_ext.h: Likewise. * db2/include/db_int.h.src: Likewise. * db2/include/hash.h: Likewise. * db2/include/hash_auto.h: Likewise. * db2/include/hash_ext.h: Likewise. * db2/include/lock.h: Likewise. * db2/include/lock_ext.h: Likewise. * db2/include/log.h: Likewise. * db2/include/log_ext.h: Likewise. * db2/include/mp.h: Likewise. * db2/include/mp_ext.h: Likewise. * db2/include/mutex_ext.h: Likewise. * db2/include/os_ext.h: Likewise. * db2/include/os_func.h: Likewise. * db2/include/txn.h: Likewise. * db2/include/txn_ext.h: Likewise. * db2/lock/lock.c: Likewise. * db2/lock/lock_deadlock.c: Likewise. * db2/log/log.c: Likewise. * db2/log/log_archive.c: Likewise. * db2/log/log_auto.c: Likewise. * db2/log/log_findckp.c: Likewise. * db2/log/log_get.c: Likewise. * db2/log/log_put.c: Likewise. * db2/log/log_rec.c: Likewise. * db2/log/log_register.c: Likewise. * db2/mp/mp_bh.c: Likewise. * db2/mp/mp_fget.c: Likewise. * db2/mp/mp_fopen.c: Likewise. * db2/mp/mp_fput.c: Likewise. * db2/mp/mp_fset.c: Likewise. * db2/mp/mp_open.c: Likewise. * db2/mp/mp_pr.c: Likewise. * db2/mp/mp_region.c: Likewise. * db2/mp/mp_sync.c: Likewise. * db2/mutex/mutex.c: Likewise. * db2/os/os_abs.c: Likewise. * db2/os/os_dir.c: Likewise. * db2/os/os_fid.c: Likewise. * db2/os/os_fsync.c: Likewise. * db2/os/os_func.c: Likewise. * db2/os/os_map.c: Likewise. * db2/os/os_oflags.c: Likewise. * db2/os/os_open.c: Likewise. * db2/os/os_rpath.c: Likewise. * db2/os/os_rw.c: Likewise. * db2/os/os_seek.c: Likewise. * db2/os/os_sleep.c: Likewise. * db2/os/os_stat.c: Likewise. * db2/os/os_unlink.c: Likewise. * db2/progs/db_deadlock/db_deadlock.c: Likewise. * db2/progs/db_dump/db_dump.c: Likewise. * db2/progs/db_load/db_load.c: Likewise. * db2/progs/db_recover/db_recover.c: Likewise. * db2/progs/db_stat/db_stat.c: Likewise. * db2/txn/txn.c: Likewise. * db2/txn/txn_auto.c: Likewise. * db2/txn/txn_rec.c: Likewise. * db2/os/db_os_abs.c: Removed. * db2/os/db_os_dir.c: Removed. * db2/os/db_os_fid.c: Removed. * db2/os/db_os_lseek.c: Removed. * db2/os/db_os_mmap.c: Removed. * db2/os/db_os_open.c: Removed. * db2/os/db_os_rw.c: Removed. * db2/os/db_os_sleep.c: Removed. * db2/os/db_os_stat.c: Removed. * db2/os/db_os_unlink.c: Removed. * libio/stdio.h (fopen): Add __restrict to parameters. * manual/process.texi (system): Describe behaviour for NULL argument. * stdio-common/printf-parse.h: Parse hh modifier. * stdio-common/vfprintf.c: Handle hh modifier. * stdio-common/vfscanf.c: Likewise. * manual/stdio.texi: Describe hh modifier for scanf/printf. * math/complex.h: Don't define _Imaginary_I, but instead _Complex_I. gcc does no yet know the `imaginary' keyword. * math/test-math.c: Add little test for know gcc bug. * math/tgmath.h: Make complex versions of log10() only available if __USE_GNU. * stdlib/test-canon.c: Fix typo. * sysdeps/generic/setenv.c: Avoid compilation warnings. Reported by Jim Meyering. * sysdeps/generic/bits/errno.h: EILSEQ is an ISO C error number. * sysdeps/mach/hurd/bits/errno.h: Likewise. * sysdeps/standalone/bits/errno.h: Likewise. * sysdeps/unix/sysv/linux/bits/errno.h: Likewise. * sysdeps/i386/i586/memcpy.S: New file. * sysdeps/i386/i586/mempcpy.S: New file. * sysdeps/i386/i586/memset.S: Fix typo. * sysdeps/posix/getcwd.c: Define HAVE_MEMPCPY for _LIBC. Add casts. * sysdeps/posix/system.c: Add comment to explain code. * sysdeps/wordsize-32/inttypes.h: Include <stddef.h> for wchar_t. Define PTRDIFF_{MIN,MAX}, SIG_ATOMIC_{MIN,MAX}, SIZE_MAX, WCHAR_{MIN,MAX}, WINT_{MIN,MAX}. Define wcstoimax, wcstoumax. * sysdeps/wordsize-64/inttypes.h: Likewise. * wcsmbs/wchar.h: Define WCHAR_{MIN,MAX} if not already defined. Declare __wcsto{l,ul,ll,ull}_internal only if not already done. * time/Makefile (routines): Add strfxtime. * time/strftime.c: Implement %F and %f format. * time/strfxtime.c: New file. * time/time.h: Define new types and symbols from ISO C 9X. * time/mktime.c: Little comment correction. 1997-11-10 Andreas Jaeger <aj@arthur.rhein-neckar.de> * sysdeps/libm-ieee754/s_sincosl.c: Fix typo. * sysdeps/libm-ieee754/s_tanl.c: Fix typo. * sysdeps/libm-ieee754/s_floorl.c: Correct typos. * sysdeps/libm-ieee754/e_remainderl.c: Replace EXTRACT_LDOUBLE_WORDS by GET_LDOUBLE_WORDS. * sysdeps/libm-ieee754/e_atan2l.c: Replace EXTRACT_LDOUBLE_WORDS by GET_LDOUBLE_WORDS. * sysdeps/libm-ieee754/s_scalbnl.c: Replace ";" by "," for correct variable declaration. * sysdeps/libm-ieee754/s_scalblnl.c: Likewise. * sysdeps/libm-ieee754/s_lrint.c (__lrint): Correct function. * math/libm-test.c (sqrt_test): Add test for sqrt (0.25). (asin_test): Add more test. 1997-11-10 23:34 Ulrich Drepper <drepper@cygnus.com> * sysdeps/libm-ieee754/e_asin.c: Add braces to make code clearer and to not confuse the poor compiler. * sysdeps/libm-ieee754/e_asinf.c: Likewise. Reported by vertex@cagent.com. 1997-11-09 Andreas Schwab <schwab@issan.informatik.uni-dortmund.de> * elf/dl-lookup.c (do_lookup): Don't accept the base version if we require a specific one. * libio/oldfreopen.c: Bind old symbols to version GLIBC_2.0. * libio/oldiofopen.c: Likewise. * libio/oldstdfiles.c: Likewise. * libc.map: Export them. 1997-11-10 07:40 H.J. Lu <hjl@gnu.ai.mit.edu> * stdlib/exit.c (exit): Handle recursive calls to exit (). 1997-11-09 Andreas Schwab <schwab@issan.informatik.uni-dortmund.de> * sysdeps/m68k/fpu/s_llrint.c: Fixed to take double argument instead of long double. * sysdeps/m68k/fpu/s_llrintf.c: New file. * sysdeps/m68k/fpu/s_llrintl.c: New file. * sysdeps/libm-ieee754/s_llrint.c: Make compilable and fix overflow condition. * sysdeps/libm-ieee754/s_llrintf.c: Fix overflow condition. * sysdeps/libm-ieee754/s_llrintl.c: Likewise. * sysdeps/libm-ieee754/s_llround.c: Likewise. * sysdeps/libm-ieee754/s_llroundf.c: Likewise. * sysdeps/libm-ieee754/s_llroundl.c: Likewise. * sysdeps/libm-ieee754/s_lrint.c: Likewise. * sysdeps/libm-ieee754/s_lrintf.c: Likewise. * sysdeps/libm-ieee754/s_lrintl.c: Likewise. * sysdeps/libm-ieee754/s_lround.c: Likewise. * sysdeps/libm-ieee754/s_lroundf.c: Likewise. * sysdeps/libm-ieee754/s_lroundl.c: Likewise. * math/libm-test.c: Test all three variants of lrint and llrint. Fix typos in lround and llround tests. Add tests for boundary cases for lrint and llround. 1997-11-08 Andreas Schwab <schwab@issan.informatik.uni-dortmund.de> * manual/arith.texi: Misc doc fixes. * manual/ctype.texi: Likewise. * manual/pattern.texi: Likewise. * manual/terminal.texi: Likewise. 1997-11-08 Andreas Schwab <schwab@issan.informatik.uni-dortmund.de> * sysdeps/unix/sysv/linux/xstatconv.c: Use struct assignment instead of memcpy to let the compiler use whatever it regards as optimal. * sysdeps/unix/sysv/linux/alpha/xstatconv.c: Likewise. 1997-11-08 Andreas Schwab <schwab@issan.informatik.uni-dortmund.de> * sysdeps/unix/sysv/linux/Makefile (sysdep_headers) [$(subdir)=misc]: Add sys/prctl.h. * sysdeps/unix/sysv/linux/Dist: Distribute it. 1997-11-08 Andreas Schwab <schwab@issan.informatik.uni-dortmund.de> * io/ftwtest-sh: Don't use the unknown which command, instead try pwd as /bin/pwd and /usr/bin/pwd. 1997-11-08 Andreas Schwab <schwab@issan.informatik.uni-dortmund.de> * manual/maint.texi (Tools for Installation): Don't recommend broken version 3.76.1 of make. (Porting): Fix wording. 1997-11-06 06:13 H.J. Lu <hjl@gnu.ai.mit.edu> * config.make.in (build-pic-default): New, defined with pic_default. * configure.in (pic_default): New, set to yes if PIC is default. * Makeconfig (CPPFLAGS-.o, CPPFLAGS-.op, CPPFLAGS-.og, CPPFLAGS-.ob): Add -DPIC if $(build-pic-default) is yes. 1997-11-09 18:15 Ulrich Drepper <drepper@cygnus.com> * Makerules (libc.so): Fix typo. * csu/Makefile (CFLAGS-initfini.s): Correctly fix moving function definition. Patch by Zack Weinberg <zack@rabi.phys.columbia.edu>. * stdlib/strtod.c: Handle numbers like 0.0e10000 correctly which produce ±0.0. Reported by Joe Keane <jgk@jgk.org>. * sysdeps/libm-ieee754/s_ceill.c: Fix typos. * sysdeps/libm-ieee754/s_llrint.c: Correct code, it never worked. 1997-11-06 07:00 H.J. Lu <hjl@gnu.ai.mit.edu> * sysdeps/unix/sysv/i386/i686/time.S: Removed. 1997-11-08 14:07 Thorsten Kukuk <kukuk@vt.uni-paderborn.de> * nis/libnsl.map: Add __do_niscall2 for nis_cachemgr. * nis/nis_call.c: Set UDP resend timeout correct. * nis/nss_compat/compat-grp.c: Rewritten to make it faster. * nis/nss_compat/compat-pwd.c: Likewise. * nis/nss_compat/compat-spwd.c: Likewise. * nis/ypclnt.c: Fix UDP resend timeout, fix yp_bind/do_ypcall interaction. * inet/protocols/routed.h: Include sys/socket.h. * inet/protocols/talkd.h: Likewise. * inet/protocols/timed.h: Include rpc/types.h. * sunrpc/rpc/pmap_clnt.h: Include rpc/clnt.h. 1997-11-06 01:39 Ulrich Drepper <drepper@cygnus.com> * Makerules (libc.so): Add missing closing brace. 1997-11-05 Brendan Kehoe <brendan@lisa.cygnus.com> * libio.h (__P): Name its arg `p' instead of `params'. This was added solely to work around problems with the definition of __P in the Solaris math.h header.
Diffstat (limited to 'db2/db')
-rw-r--r--db2/db/db.c30
-rw-r--r--db2/db/db.src10
-rw-r--r--db2/db/db_auto.c75
-rw-r--r--db2/db/db_dispatch.c14
-rw-r--r--db2/db/db_dup.c4
-rw-r--r--db2/db/db_overflow.c32
-rw-r--r--db2/db/db_pr.c9
-rw-r--r--db2/db/db_rec.c21
-rw-r--r--db2/db/db_ret.c8
-rw-r--r--db2/db/db_thread.c10
10 files changed, 114 insertions, 99 deletions
diff --git a/db2/db/db.c b/db2/db/db.c
index 9ebe73cf6e..50b14eba7c 100644
--- a/db2/db/db.c
+++ b/db2/db/db.c
@@ -44,7 +44,7 @@
#include "config.h"
#ifndef lint
-static const char sccsid[] = "@(#)db.c 10.41 (Sleepycat) 9/23/97";
+static const char sccsid[] = "@(#)db.c 10.44 (Sleepycat) 10/25/97";
#endif /* not lint */
#ifndef NO_SYSTEM_INCLUDES
@@ -125,13 +125,19 @@ db_open(fname, type, flags, mode, dbenv, dbinfo, dbpp)
if ((ret = __db_fchk(dbenv, "db_open", flags, OKFLAGS)) != 0)
return (ret);
+ if (dbenv != NULL &&
+ LF_ISSET(DB_THREAD) && !F_ISSET(dbenv, DB_ENV_THREAD)) {
+ __db_err(dbenv, "environment not created using DB_THREAD");
+ return (EINVAL);
+ }
+
/* Initialize for error return. */
fd = -1;
need_fileid = 1;
real_name = NULL;
/* Allocate the DB structure, reference the DB_ENV structure. */
- if ((dbp = (DB *)calloc(1, sizeof(DB))) == NULL) {
+ if ((dbp = (DB *)__db_calloc(1, sizeof(DB))) == NULL) {
__db_err(dbenv, "%s", strerror(ENOMEM));
return (ENOMEM);
}
@@ -239,7 +245,7 @@ db_open(fname, type, flags, mode, dbenv, dbinfo, dbpp)
*/
retry_cnt = 0;
open_retry: if (LF_ISSET(DB_CREATE)) {
- if ((ret = __db_fdopen(real_name, flags | DB_EXCL,
+ if ((ret = __db_open(real_name, flags | DB_EXCL,
OKFLAGS | DB_EXCL, mode, &fd)) != 0)
if (ret == EEXIST) {
LF_CLR(DB_CREATE);
@@ -250,7 +256,7 @@ open_retry: if (LF_ISSET(DB_CREATE)) {
goto err;
}
} else
- if ((ret = __db_fdopen(real_name,
+ if ((ret = __db_open(real_name,
flags, OKFLAGS, mode, &fd)) != 0) {
__db_err(dbenv, "%s: %s", fname, strerror(ret));
goto err;
@@ -264,8 +270,11 @@ open_retry: if (LF_ISSET(DB_CREATE)) {
*/
if (dbp->pgsize == 0) {
if ((ret =
- __db_stat(dbenv, real_name, fd, NULL, &io)) != 0)
+ __db_ioinfo(real_name, fd, NULL, &io)) != 0) {
+ __db_err(dbenv,
+ "%s: %s", real_name, strerror(ret));
goto err;
+ }
if (io < 512)
io = 512;
if (io > 16 * 1024)
@@ -477,7 +486,7 @@ empty: /*
if (dbenv == NULL) {
if ((dbp->mp_dbenv =
- (DB_ENV *)calloc(sizeof(DB_ENV), 1)) == NULL) {
+ (DB_ENV *)__db_calloc(sizeof(DB_ENV), 1)) == NULL) {
ret = ENOMEM;
goto err;
}
@@ -491,9 +500,9 @@ empty: /*
restore = 1;
}
envp->mp_size = cachesize;
- F_SET(envp, DB_MPOOL_PRIVATE);
- if ((ret = memp_open(NULL,
- DB_CREATE, S_IRUSR | S_IWUSR, envp, &dbp->mp)) != 0)
+ if ((ret = memp_open(NULL, DB_CREATE | DB_MPOOL_PRIVATE |
+ (F_ISSET(dbp, DB_AM_THREAD) ? DB_THREAD : 0),
+ S_IRUSR | S_IWUSR, envp, &dbp->mp)) != 0)
goto err;
if (restore)
*dbenv = t_dbenv;
@@ -725,7 +734,8 @@ db_close(dbp, flags)
}
/* Sync the memory pool. */
- if ((t_ret = memp_fsync(dbp->mpf)) != 0 && ret == 0)
+ if ((t_ret = memp_fsync(dbp->mpf)) != 0 &&
+ t_ret != DB_INCOMPLETE && ret == 0)
ret = t_ret;
/* Close the memory pool file. */
diff --git a/db2/db/db.src b/db2/db/db.src
index a3e2f7b75c..07d98123ac 100644
--- a/db2/db/db.src
+++ b/db2/db/db.src
@@ -3,7 +3,7 @@
*
* Copyright (c) 1996, 1997
* Sleepycat Software. All rights reserved.
- * @(#)db.src 10.3 (Sleepycat) 8/18/97
+ * @(#)db.src 10.4 (Sleepycat) 11/2/97
*/
#include "config.h"
@@ -81,15 +81,17 @@ POINTER nextlsn DB_LSN * lu
END
/*
- * ovref -- Handles increment of overflow page reference count.
+ * ovref -- Handles increment/decrement of overflow page reference count.
*
* fileid: identifies the file being modified.
- * pgno: page number being incremented.
- * lsn the page's original lsn.
+ * pgno: page number whose ref count is being incremented/decremented.
+ * adjust: the adjustment being made.
+ * lsn: the page's original lsn.
*/
BEGIN ovref
ARG fileid u_int32_t lu
ARG pgno db_pgno_t lu
+ARG adjust int32_t ld
POINTER lsn DB_LSN * lu
END
diff --git a/db2/db/db_auto.c b/db2/db/db_auto.c
index 6922504383..d40d964542 100644
--- a/db2/db/db_auto.c
+++ b/db2/db/db_auto.c
@@ -62,7 +62,7 @@ int __db_addrem_log(logp, txnid, ret_lsnp, flags,
+ sizeof(u_int32_t) + (hdr == NULL ? 0 : hdr->size)
+ sizeof(u_int32_t) + (dbt == NULL ? 0 : dbt->size)
+ sizeof(*pagelsn);
- if ((logrec.data = (void *)malloc(logrec.size)) == NULL)
+ if ((logrec.data = (void *)__db_malloc(logrec.size)) == NULL)
return (ENOMEM);
bp = logrec.data;
@@ -114,7 +114,7 @@ int __db_addrem_log(logp, txnid, ret_lsnp, flags,
ret = log_put(logp, ret_lsnp, (DBT *)&logrec, flags);
if (txnid != NULL)
txnid->last_lsn = *ret_lsnp;
- free(logrec.data);
+ __db_free(logrec.data);
return (ret);
}
@@ -176,7 +176,7 @@ __db_addrem_print(notused1, dbtp, lsnp, notused3, notused4)
printf("\tpagelsn: [%lu][%lu]\n",
(u_long)argp->pagelsn.file, (u_long)argp->pagelsn.offset);
printf("\n");
- free(argp);
+ __db_free(argp);
return (0);
}
@@ -191,7 +191,7 @@ __db_addrem_read(recbuf, argpp)
__db_addrem_args *argp;
u_int8_t *bp;
- argp = (__db_addrem_args *)malloc(sizeof(__db_addrem_args) +
+ argp = (__db_addrem_args *)__db_malloc(sizeof(__db_addrem_args) +
sizeof(DB_TXN));
if (argp == NULL)
return (ENOMEM);
@@ -266,7 +266,7 @@ int __db_split_log(logp, txnid, ret_lsnp, flags,
+ sizeof(pgno)
+ sizeof(u_int32_t) + (pageimage == NULL ? 0 : pageimage->size)
+ sizeof(*pagelsn);
- if ((logrec.data = (void *)malloc(logrec.size)) == NULL)
+ if ((logrec.data = (void *)__db_malloc(logrec.size)) == NULL)
return (ENOMEM);
bp = logrec.data;
@@ -304,7 +304,7 @@ int __db_split_log(logp, txnid, ret_lsnp, flags,
ret = log_put(logp, ret_lsnp, (DBT *)&logrec, flags);
if (txnid != NULL)
txnid->last_lsn = *ret_lsnp;
- free(logrec.data);
+ __db_free(logrec.data);
return (ret);
}
@@ -355,7 +355,7 @@ __db_split_print(notused1, dbtp, lsnp, notused3, notused4)
printf("\tpagelsn: [%lu][%lu]\n",
(u_long)argp->pagelsn.file, (u_long)argp->pagelsn.offset);
printf("\n");
- free(argp);
+ __db_free(argp);
return (0);
}
@@ -370,7 +370,7 @@ __db_split_read(recbuf, argpp)
__db_split_args *argp;
u_int8_t *bp;
- argp = (__db_split_args *)malloc(sizeof(__db_split_args) +
+ argp = (__db_split_args *)__db_malloc(sizeof(__db_split_args) +
sizeof(DB_TXN));
if (argp == NULL)
return (ENOMEM);
@@ -447,7 +447,7 @@ int __db_big_log(logp, txnid, ret_lsnp, flags,
+ sizeof(*pagelsn)
+ sizeof(*prevlsn)
+ sizeof(*nextlsn);
- if ((logrec.data = (void *)malloc(logrec.size)) == NULL)
+ if ((logrec.data = (void *)__db_malloc(logrec.size)) == NULL)
return (ENOMEM);
bp = logrec.data;
@@ -499,7 +499,7 @@ int __db_big_log(logp, txnid, ret_lsnp, flags,
ret = log_put(logp, ret_lsnp, (DBT *)&logrec, flags);
if (txnid != NULL)
txnid->last_lsn = *ret_lsnp;
- free(logrec.data);
+ __db_free(logrec.data);
return (ret);
}
@@ -556,7 +556,7 @@ __db_big_print(notused1, dbtp, lsnp, notused3, notused4)
printf("\tnextlsn: [%lu][%lu]\n",
(u_long)argp->nextlsn.file, (u_long)argp->nextlsn.offset);
printf("\n");
- free(argp);
+ __db_free(argp);
return (0);
}
@@ -571,7 +571,7 @@ __db_big_read(recbuf, argpp)
__db_big_args *argp;
u_int8_t *bp;
- argp = (__db_big_args *)malloc(sizeof(__db_big_args) +
+ argp = (__db_big_args *)__db_malloc(sizeof(__db_big_args) +
sizeof(DB_TXN));
if (argp == NULL)
return (ENOMEM);
@@ -610,16 +610,17 @@ __db_big_read(recbuf, argpp)
/*
* PUBLIC: int __db_ovref_log
* PUBLIC: __P((DB_LOG *, DB_TXN *, DB_LSN *, u_int32_t,
- * PUBLIC: u_int32_t, db_pgno_t, DB_LSN *));
+ * PUBLIC: u_int32_t, db_pgno_t, int32_t, DB_LSN *));
*/
int __db_ovref_log(logp, txnid, ret_lsnp, flags,
- fileid, pgno, lsn)
+ fileid, pgno, adjust, lsn)
DB_LOG *logp;
DB_TXN *txnid;
DB_LSN *ret_lsnp;
u_int32_t flags;
u_int32_t fileid;
db_pgno_t pgno;
+ int32_t adjust;
DB_LSN * lsn;
{
DBT logrec;
@@ -639,8 +640,9 @@ int __db_ovref_log(logp, txnid, ret_lsnp, flags,
logrec.size = sizeof(rectype) + sizeof(txn_num) + sizeof(DB_LSN)
+ sizeof(fileid)
+ sizeof(pgno)
+ + sizeof(adjust)
+ sizeof(*lsn);
- if ((logrec.data = (void *)malloc(logrec.size)) == NULL)
+ if ((logrec.data = (void *)__db_malloc(logrec.size)) == NULL)
return (ENOMEM);
bp = logrec.data;
@@ -654,6 +656,8 @@ int __db_ovref_log(logp, txnid, ret_lsnp, flags,
bp += sizeof(fileid);
memcpy(bp, &pgno, sizeof(pgno));
bp += sizeof(pgno);
+ memcpy(bp, &adjust, sizeof(adjust));
+ bp += sizeof(adjust);
if (lsn != NULL)
memcpy(bp, lsn, sizeof(*lsn));
else
@@ -666,7 +670,7 @@ int __db_ovref_log(logp, txnid, ret_lsnp, flags,
ret = log_put(logp, ret_lsnp, (DBT *)&logrec, flags);
if (txnid != NULL)
txnid->last_lsn = *ret_lsnp;
- free(logrec.data);
+ __db_free(logrec.data);
return (ret);
}
@@ -704,10 +708,11 @@ __db_ovref_print(notused1, dbtp, lsnp, notused3, notused4)
(u_long)argp->prev_lsn.offset);
printf("\tfileid: %lu\n", (u_long)argp->fileid);
printf("\tpgno: %lu\n", (u_long)argp->pgno);
+ printf("\tadjust: %ld\n", (long)argp->adjust);
printf("\tlsn: [%lu][%lu]\n",
(u_long)argp->lsn.file, (u_long)argp->lsn.offset);
printf("\n");
- free(argp);
+ __db_free(argp);
return (0);
}
@@ -722,7 +727,7 @@ __db_ovref_read(recbuf, argpp)
__db_ovref_args *argp;
u_int8_t *bp;
- argp = (__db_ovref_args *)malloc(sizeof(__db_ovref_args) +
+ argp = (__db_ovref_args *)__db_malloc(sizeof(__db_ovref_args) +
sizeof(DB_TXN));
if (argp == NULL)
return (ENOMEM);
@@ -738,6 +743,8 @@ __db_ovref_read(recbuf, argpp)
bp += sizeof(argp->fileid);
memcpy(&argp->pgno, bp, sizeof(argp->pgno));
bp += sizeof(argp->pgno);
+ memcpy(&argp->adjust, bp, sizeof(argp->adjust));
+ bp += sizeof(argp->adjust);
memcpy(&argp->lsn, bp, sizeof(argp->lsn));
bp += sizeof(argp->lsn);
*argpp = argp;
@@ -787,7 +794,7 @@ int __db_relink_log(logp, txnid, ret_lsnp, flags,
+ sizeof(*lsn_prev)
+ sizeof(next)
+ sizeof(*lsn_next);
- if ((logrec.data = (void *)malloc(logrec.size)) == NULL)
+ if ((logrec.data = (void *)__db_malloc(logrec.size)) == NULL)
return (ENOMEM);
bp = logrec.data;
@@ -827,7 +834,7 @@ int __db_relink_log(logp, txnid, ret_lsnp, flags,
ret = log_put(logp, ret_lsnp, (DBT *)&logrec, flags);
if (txnid != NULL)
txnid->last_lsn = *ret_lsnp;
- free(logrec.data);
+ __db_free(logrec.data);
return (ret);
}
@@ -874,7 +881,7 @@ __db_relink_print(notused1, dbtp, lsnp, notused3, notused4)
printf("\tlsn_next: [%lu][%lu]\n",
(u_long)argp->lsn_next.file, (u_long)argp->lsn_next.offset);
printf("\n");
- free(argp);
+ __db_free(argp);
return (0);
}
@@ -889,7 +896,7 @@ __db_relink_read(recbuf, argpp)
__db_relink_args *argp;
u_int8_t *bp;
- argp = (__db_relink_args *)malloc(sizeof(__db_relink_args) +
+ argp = (__db_relink_args *)__db_malloc(sizeof(__db_relink_args) +
sizeof(DB_TXN));
if (argp == NULL)
return (ENOMEM);
@@ -957,7 +964,7 @@ int __db_addpage_log(logp, txnid, ret_lsnp, flags,
+ sizeof(*lsn)
+ sizeof(nextpgno)
+ sizeof(*nextlsn);
- if ((logrec.data = (void *)malloc(logrec.size)) == NULL)
+ if ((logrec.data = (void *)__db_malloc(logrec.size)) == NULL)
return (ENOMEM);
bp = logrec.data;
@@ -990,7 +997,7 @@ int __db_addpage_log(logp, txnid, ret_lsnp, flags,
ret = log_put(logp, ret_lsnp, (DBT *)&logrec, flags);
if (txnid != NULL)
txnid->last_lsn = *ret_lsnp;
- free(logrec.data);
+ __db_free(logrec.data);
return (ret);
}
@@ -1034,7 +1041,7 @@ __db_addpage_print(notused1, dbtp, lsnp, notused3, notused4)
printf("\tnextlsn: [%lu][%lu]\n",
(u_long)argp->nextlsn.file, (u_long)argp->nextlsn.offset);
printf("\n");
- free(argp);
+ __db_free(argp);
return (0);
}
@@ -1049,7 +1056,7 @@ __db_addpage_read(recbuf, argpp)
__db_addpage_args *argp;
u_int8_t *bp;
- argp = (__db_addpage_args *)malloc(sizeof(__db_addpage_args) +
+ argp = (__db_addpage_args *)__db_malloc(sizeof(__db_addpage_args) +
sizeof(DB_TXN));
if (argp == NULL)
return (ENOMEM);
@@ -1114,7 +1121,7 @@ int __db_debug_log(logp, txnid, ret_lsnp, flags,
+ sizeof(u_int32_t) + (key == NULL ? 0 : key->size)
+ sizeof(u_int32_t) + (data == NULL ? 0 : data->size)
+ sizeof(arg_flags);
- if ((logrec.data = (void *)malloc(logrec.size)) == NULL)
+ if ((logrec.data = (void *)__db_malloc(logrec.size)) == NULL)
return (ENOMEM);
bp = logrec.data;
@@ -1165,7 +1172,7 @@ int __db_debug_log(logp, txnid, ret_lsnp, flags,
ret = log_put(logp, ret_lsnp, (DBT *)&logrec, flags);
if (txnid != NULL)
txnid->last_lsn = *ret_lsnp;
- free(logrec.data);
+ __db_free(logrec.data);
return (ret);
}
@@ -1231,7 +1238,7 @@ __db_debug_print(notused1, dbtp, lsnp, notused3, notused4)
printf("\n");
printf("\targ_flags: %lu\n", (u_long)argp->arg_flags);
printf("\n");
- free(argp);
+ __db_free(argp);
return (0);
}
@@ -1246,7 +1253,7 @@ __db_debug_read(recbuf, argpp)
__db_debug_args *argp;
u_int8_t *bp;
- argp = (__db_debug_args *)malloc(sizeof(__db_debug_args) +
+ argp = (__db_debug_args *)__db_malloc(sizeof(__db_debug_args) +
sizeof(DB_TXN));
if (argp == NULL)
return (ENOMEM);
@@ -1303,7 +1310,7 @@ int __db_noop_log(logp, txnid, ret_lsnp, flags)
} else
lsnp = &txnid->last_lsn;
logrec.size = sizeof(rectype) + sizeof(txn_num) + sizeof(DB_LSN);
- if ((logrec.data = (void *)malloc(logrec.size)) == NULL)
+ if ((logrec.data = (void *)__db_malloc(logrec.size)) == NULL)
return (ENOMEM);
bp = logrec.data;
@@ -1320,7 +1327,7 @@ int __db_noop_log(logp, txnid, ret_lsnp, flags)
ret = log_put(logp, ret_lsnp, (DBT *)&logrec, flags);
if (txnid != NULL)
txnid->last_lsn = *ret_lsnp;
- free(logrec.data);
+ __db_free(logrec.data);
return (ret);
}
@@ -1357,7 +1364,7 @@ __db_noop_print(notused1, dbtp, lsnp, notused3, notused4)
(u_long)argp->prev_lsn.file,
(u_long)argp->prev_lsn.offset);
printf("\n");
- free(argp);
+ __db_free(argp);
return (0);
}
@@ -1372,7 +1379,7 @@ __db_noop_read(recbuf, argpp)
__db_noop_args *argp;
u_int8_t *bp;
- argp = (__db_noop_args *)malloc(sizeof(__db_noop_args) +
+ argp = (__db_noop_args *)__db_malloc(sizeof(__db_noop_args) +
sizeof(DB_TXN));
if (argp == NULL)
return (ENOMEM);
diff --git a/db2/db/db_dispatch.c b/db2/db/db_dispatch.c
index 3d7b162d75..a4bcdb7628 100644
--- a/db2/db/db_dispatch.c
+++ b/db2/db/db_dispatch.c
@@ -43,7 +43,7 @@
#include "config.h"
#ifndef lint
-static const char sccsid[] = "@(#)db_dispatch.c 10.5 (Sleepycat) 7/2/97";
+static const char sccsid[] = "@(#)db_dispatch.c 10.6 (Sleepycat) 10/25/97";
#endif /* not lint */
#ifndef NO_SYSTEM_INCLUDES
@@ -155,12 +155,12 @@ __db_add_recovery(dbenv, func, ndx)
if (ndx >= dispatch_size) {
if (dispatch_table == NULL)
dispatch_table = (int (**)
- __P((DB_LOG *, DBT *, DB_LSN *, int, void *)))
- malloc(DB_user_BEGIN * sizeof(dispatch_table[0]));
+ __P((DB_LOG *, DBT *, DB_LSN *, int, void *)))
+ __db_malloc(DB_user_BEGIN * sizeof(dispatch_table[0]));
else
dispatch_table = (int (**)
__P((DB_LOG *, DBT *, DB_LSN *, int, void *)))
- realloc(dispatch_table, (DB_user_BEGIN +
+ __db_realloc(dispatch_table, (DB_user_BEGIN +
dispatch_size) * sizeof(dispatch_table[0]));
if (dispatch_table == NULL) {
__db_err(dbenv, "%s", strerror(ENOMEM));
@@ -187,8 +187,8 @@ __db_txnlist_init(retp)
{
__db_txnhead *headp;
- if ((headp =
- (struct __db_txnhead *)malloc(sizeof(struct __db_txnhead))) == NULL)
+ if ((headp = (struct __db_txnhead *)
+ __db_malloc(sizeof(struct __db_txnhead))) == NULL)
return (ENOMEM);
LIST_INIT(&headp->head);
@@ -212,7 +212,7 @@ __db_txnlist_add(listp, txnid)
__db_txnhead *hp;
__db_txnlist *elp;
- if ((elp = (__db_txnlist *)malloc(sizeof(__db_txnlist))) == NULL)
+ if ((elp = (__db_txnlist *)__db_malloc(sizeof(__db_txnlist))) == NULL)
return (ENOMEM);
elp->txnid = txnid;
diff --git a/db2/db/db_dup.c b/db2/db/db_dup.c
index 66c6c2616a..faeefa0744 100644
--- a/db2/db/db_dup.c
+++ b/db2/db/db_dup.c
@@ -8,7 +8,7 @@
#include "config.h"
#ifndef lint
-static const char sccsid[] = "@(#)db_dup.c 10.9 (Sleepycat) 9/3/97";
+static const char sccsid[] = "@(#)db_dup.c 10.10 (Sleepycat) 10/25/97";
#endif /* not lint */
#ifndef NO_SYSTEM_INCLUDES
@@ -217,7 +217,7 @@ __db_dsplit(dbp, hp, indxp, size, newfunc)
indx = *indxp;
/* Create a temporary page to do compaction onto. */
- if ((tp = (PAGE *)malloc(dbp->pgsize)) == NULL)
+ if ((tp = (PAGE *)__db_malloc(dbp->pgsize)) == NULL)
return (ENOMEM);
#ifdef DEBUG
memset(tp, 0xff, dbp->pgsize);
diff --git a/db2/db/db_pr.c b/db2/db/db_pr.c
index 09d8057da4..6b6171a13c 100644
--- a/db2/db/db_pr.c
+++ b/db2/db/db_pr.c
@@ -8,7 +8,7 @@
#include "config.h"
#ifndef lint
-static const char sccsid[] = "@(#)db_pr.c 10.17 (Sleepycat) 9/15/97";
+static const char sccsid[] = "@(#)db_pr.c 10.19 (Sleepycat) 11/2/97";
#endif /* not lint */
#ifndef NO_SYSTEM_INCLUDES
@@ -183,7 +183,6 @@ __db_prbtree(dbp)
};
BTMETA *mp;
BTREE *t;
- DB_LOCK lock;
EPG *epg;
FILE *fp;
RECNO *rp;
@@ -195,8 +194,6 @@ __db_prbtree(dbp)
(void)fprintf(fp, "%s\nOn-page metadata:\n", DB_LINE);
i = PGNO_METADATA;
- if ((ret = __bam_lget(dbp, 0, PGNO_METADATA, DB_LOCK_READ, &lock)) != 0)
- return (ret);
if ((ret = __bam_pget(dbp, (PAGE **)&mp, &i, 0)) != 0)
return (ret);
@@ -211,7 +208,6 @@ __db_prbtree(dbp)
__db_prflags(mp->flags, mfn);
(void)fprintf(fp, "\n");
(void)memp_fput(dbp->mpf, mp, 0);
- (void)__bam_lput(dbp, lock);
(void)fprintf(fp, "%s\nDB_INFO:\n", DB_LINE);
(void)fprintf(fp, "bt_maxkey: %lu bt_minkey: %lu\n",
@@ -416,7 +412,8 @@ __db_prpage(h, all)
(TYPE(h) == P_LRECNO && h->pgno == PGNO_ROOT))
fprintf(fp, " total records: %4lu", (u_long)RE_NREC(h));
fprintf(fp, "\n");
- if (TYPE(h) == P_LBTREE || TYPE(h) == P_LRECNO)
+ if (TYPE(h) == P_LBTREE || TYPE(h) == P_LRECNO ||
+ TYPE(h) == P_DUPLICATE || TYPE(h) == P_OVERFLOW)
fprintf(fp, " prev: %4lu next: %4lu",
(u_long)PREV_PGNO(h), (u_long)NEXT_PGNO(h));
if (TYPE(h) == P_IBTREE || TYPE(h) == P_LBTREE)
diff --git a/db2/db/db_rec.c b/db2/db/db_rec.c
index 900b0ed579..2c9ca9abe0 100644
--- a/db2/db/db_rec.c
+++ b/db2/db/db_rec.c
@@ -8,7 +8,7 @@
#include "config.h"
#ifndef lint
-static const char sccsid[] = "@(#)db_rec.c 10.8 (Sleepycat) 8/22/97";
+static const char sccsid[] = "@(#)db_rec.c 10.10 (Sleepycat) 11/2/97";
#endif /* not lint */
#ifndef NO_SYSTEM_INCLUDES
@@ -330,7 +330,7 @@ out: REC_CLOSE;
/*
* __db_ovref_recover --
- * Recovery function for __db_ioff().
+ * Recovery function for __db_ovref().
*
* PUBLIC: int __db_ovref_recover __P((DB_LOG *, DBT *, DB_LSN *, int, void *));
*/
@@ -357,22 +357,21 @@ __db_ovref_recover(logp, dbtp, lsnp, redo, info)
}
modified = 0;
- if (log_compare(lsnp, &argp->lsn) == 0 && redo) {
+ if (log_compare(&LSN(pagep), &argp->lsn) == 0 && redo) {
/* Need to redo update described. */
- ++OV_REF(pagep);
+ OV_REF(pagep) += argp->adjust;
pagep->lsn = *lsnp;
modified = 1;
} else if (log_compare(lsnp, &LSN(pagep)) == 0 && !redo) {
/* Need to undo update described. */
- --OV_REF(pagep);
+ OV_REF(pagep) -= argp->adjust;
pagep->lsn = argp->lsn;
modified = 1;
}
- ret = memp_fput(mpf, pagep, modified ? DB_MPOOL_DIRTY : 0);
-
- *lsnp = argp->prev_lsn;
+ if ((ret = memp_fput(mpf, pagep, modified ? DB_MPOOL_DIRTY : 0)) == 0)
+ *lsnp = argp->prev_lsn;
out: REC_CLOSE;
}
@@ -413,7 +412,7 @@ __db_relink_recover(logp, dbtp, lsnp, redo, info)
goto next;
}
modified = 0;
- if (log_compare(lsnp, &argp->lsn) == 0 && redo) {
+ if (log_compare(&LSN(pagep), &argp->lsn) == 0 && redo) {
/* Redo the relink. */
pagep->lsn = *lsnp;
modified = 1;
@@ -438,7 +437,7 @@ next: if ((ret = memp_fget(mpf, &argp->next, 0, &pagep)) != 0) {
goto prev;
}
modified = 0;
- if (log_compare(lsnp, &argp->lsn_next) == 0 && redo) {
+ if (log_compare(&LSN(pagep), &argp->lsn_next) == 0 && redo) {
/* Redo the relink. */
pagep->prev_pgno = argp->prev;
@@ -464,7 +463,7 @@ prev: if ((ret = memp_fget(mpf, &argp->prev, 0, &pagep)) != 0) {
goto done;
}
modified = 0;
- if (log_compare(lsnp, &argp->lsn_prev) == 0 && redo) {
+ if (log_compare(&LSN(pagep), &argp->lsn_prev) == 0 && redo) {
/* Redo the relink. */
pagep->next_pgno = argp->next;
diff --git a/db2/db/db_ret.c b/db2/db/db_ret.c
index ee2bc82f87..bcec308b95 100644
--- a/db2/db/db_ret.c
+++ b/db2/db/db_ret.c
@@ -8,7 +8,7 @@
#include "config.h"
#ifndef lint
-static const char sccsid[] = "@(#)db_ret.c 10.7 (Sleepycat) 9/15/97";
+static const char sccsid[] = "@(#)db_ret.c 10.8 (Sleepycat) 10/25/97";
#endif /* not lint */
#ifndef NO_SYSTEM_INCLUDES
@@ -122,7 +122,7 @@ __db_retcopy(dbt, data, len, memp, memsize, db_malloc)
*/
if (F_ISSET(dbt, DB_DBT_MALLOC)) {
dbt->data = db_malloc == NULL ?
- (void *)malloc(len + 1) :
+ (void *)__db_malloc(len + 1) :
(void *)db_malloc(len + 1);
if (dbt->data == NULL)
return (ENOMEM);
@@ -134,8 +134,8 @@ __db_retcopy(dbt, data, len, memp, memsize, db_malloc)
} else {
if (*memsize == 0 || *memsize < len) {
*memp = *memp == NULL ?
- (void *)malloc(len + 1) :
- (void *)realloc(*memp, len + 1);
+ (void *)__db_malloc(len + 1) :
+ (void *)__db_realloc(*memp, len + 1);
if (*memp == NULL) {
*memsize = 0;
return (ENOMEM);
diff --git a/db2/db/db_thread.c b/db2/db/db_thread.c
index 170baf5345..d9086918dd 100644
--- a/db2/db/db_thread.c
+++ b/db2/db/db_thread.c
@@ -8,7 +8,7 @@
#include "config.h"
#ifndef lint
-static const char sccsid[] = "@(#)db_thread.c 8.12 (Sleepycat) 9/23/97";
+static const char sccsid[] = "@(#)db_thread.c 8.13 (Sleepycat) 10/25/97";
#endif /* not lint */
#ifndef NO_SYSTEM_INCLUDES
@@ -42,8 +42,7 @@ __db_gethandle(dbp, am_func, dbpp)
DB *ret_dbp;
int ret, t_ret;
- if ((ret = __db_mutex_lock((db_mutex_t *)dbp->mutexp, -1,
- dbp->dbenv == NULL ? NULL : dbp->dbenv->db_yield)) != 0)
+ if ((ret = __db_mutex_lock((db_mutex_t *)dbp->mutexp, -1)) != 0)
return (ret);
if ((ret_dbp = LIST_FIRST(&dbp->handleq)) != NULL)
@@ -51,7 +50,7 @@ __db_gethandle(dbp, am_func, dbpp)
LIST_REMOVE(ret_dbp, links);
else {
/* Allocate a new handle. */
- if ((ret_dbp = (DB *)malloc(sizeof(*dbp))) == NULL) {
+ if ((ret_dbp = (DB *)__db_malloc(sizeof(*dbp))) == NULL) {
ret = ENOMEM;
goto err;
}
@@ -94,8 +93,7 @@ __db_puthandle(dbp)
int ret;
master = dbp->master;
- if ((ret = __db_mutex_lock((db_mutex_t *)master->mutexp, -1,
- dbp->dbenv == NULL ? NULL : dbp->dbenv->db_yield)) != 0)
+ if ((ret = __db_mutex_lock((db_mutex_t *)master->mutexp, -1)) != 0)
return (ret);
LIST_INSERT_HEAD(&master->handleq, dbp, links);
diff --git a/db2/db/db_overflow.c b/db2/db/db_overflow.c
index 2340e9e358..8c6619f228 100644
--- a/db2/db/db_overflow.c
+++ b/db2/db/db_overflow.c
@@ -47,7 +47,7 @@
#include "config.h"
#ifndef lint
-static const char sccsid[] = "@(#)db_overflow.c 10.4 (Sleepycat) 7/2/97";
+static const char sccsid[] = "@(#)db_overflow.c 10.7 (Sleepycat) 11/2/97";
#endif /* not lint */
#ifndef NO_SYSTEM_INCLUDES
@@ -121,14 +121,14 @@ __db_goff(dbp, dbt, tlen, pgno, bpp, bpsz)
}
} else if (F_ISSET(dbt, DB_DBT_MALLOC)) {
dbt->data = dbp->db_malloc == NULL ?
- (void *)malloc(needed + 1) :
+ (void *)__db_malloc(needed + 1) :
(void *)dbp->db_malloc(needed + 1);
if (dbt->data == NULL)
return (ENOMEM);
} else if (*bpsz == 0 || *bpsz < needed) {
*bpp = (*bpp == NULL ?
- (void *)malloc(needed + 1) :
- (void *)realloc(*bpp, needed + 1));
+ (void *)__db_malloc(needed + 1) :
+ (void *)__db_realloc(*bpp, needed + 1));
if (*bpp == NULL)
return (ENOMEM);
*bpsz = needed + 1;
@@ -256,15 +256,16 @@ __db_poff(dbp, dbt, pgnop, newfunc)
}
/*
* __db_ioff --
* Increment the reference count on an overflow page.
*
- * PUBLIC: int __db_ioff __P((DB *, db_pgno_t));
+ * PUBLIC: int __db_ovref __P((DB *, db_pgno_t, int));
*/
int
-__db_ioff(dbp, pgno)
+__db_ovref(dbp, pgno, adjust)
DB *dbp;
db_pgno_t pgno;
+ int adjust;
{
PAGE *h;
int ret;
@@ -274,10 +275,12 @@ __db_ioff(dbp, pgno)
return (ret);
}
- ++OV_REF(h);
- if (DB_LOGGING(dbp) && (ret = __db_ovref_log(dbp->dbenv->lg_info,
- dbp->txn, &LSN(h), 0, dbp->log_fileid, h->pgno, &LSN(h))) != 0)
- return (ret);
+ if (DB_LOGGING(dbp))
+ if ((ret = __db_ovref_log(dbp->dbenv->lg_info, dbp->txn,
+ &LSN(h), 0, dbp->log_fileid, h->pgno, (int32_t)adjust,
+ &LSN(h))) != 0)
+ return (ret);
+ OV_REF(h) += adjust;
(void)memp_fput(dbp->mpf, h, DB_MPOOL_DIRTY);
return (0);
@@ -311,9 +314,8 @@ __db_doff(dbp, pgno, freefunc)
* one key/data item, decrement the reference count and return.
*/
if (TYPE(pagep) == P_OVERFLOW && OV_REF(pagep) > 1) {
- --OV_REF(pagep);
- (void)memp_fput(dbp->mpf, pagep, DB_MPOOL_DIRTY);
- return (0);
+ (void)memp_fput(dbp->mpf, pagep, 0);
+ return (__db_ovref(dbp, pgno, -1));
}
if (DB_LOGGING(dbp)) {
@@ -256,15 +256,16 @@ __db_poff(dbp, dbt, pgnop, newfunc)
}
/*
- * __db_ioff --
- * Increment the reference count on an overflow page.
+ * __db_ovref --
+ * Increment/decrement the reference count on an overflow page.
*